@@ -1,6 +1,6 @@
 package WebDAO::Container;
 
-#$Id$
+#$Id: Container.pm 852 2010-10-17 18:57:02Z zag $
 
 =head1 NAME
 
@@ -77,7 +77,7 @@ Insert set of objects into container
 
 sub _add_childs_ {
     my $self = shift;
-    $self->__add_childs__( 1, @_ );
+    $self->__add_childs__(1, @_);
 }
 
 #deprecated
@@ -110,24 +110,25 @@ Clear all childs (except "pre" and "post" objects), and set  to C<@childs_set>
 sub __set_childs__ {
     my $self = shift;
     my $type = shift;
-    my $dst  = $type == 0
-      ? $self->__pre_childs    #0
-      : $type == 1 ? $self->__childs()        #1
-      :              $self->__post_childs;    #2
-    for ( @{$dst} ) {
+    my $dst =
+        $type == 0 ? $self->__pre_childs #0
+      : $type == 1 ? $self->__childs()   #1 
+      :              $self->__post_childs; #2
+    for ( @{ $dst} ) {
         $_->_destroy;
     }
-    $self->__add_childs__( $type, @_ );
+    $self->__add_childs__($type, @_)    
 }
 
+
 # 0 - pre, 1 - fetch , 2 - post
 sub __add_childs__ {
     my $self = shift;
     my $type = shift;
-    my $dst  = $type == 0
-      ? $self->__pre_childs                   #0
-      : $type == 1 ? $self->__childs()        #1
-      :              $self->__post_childs;    #2
+    my $dst =
+        $type == 0 ? $self->__pre_childs #0
+      : $type == 1 ? $self->__childs()   #1 
+      :              $self->__post_childs; #2
     my @childs =
       grep { ref $_ }
       map { ref($_) eq 'ARRAY' ? @$_ : $_ }
@@ -139,7 +140,7 @@ sub __add_childs__ {
         $_->_set_parent($self) for @childs;
         $self->getEngine->__restore_session_attributes(@childs);
     }
-    push( @{$dst}, @childs );
+    push( @{ $dst }, @childs );
 }
 
 sub _set_childs_ {
@@ -163,7 +164,7 @@ Output data precede to fetch method. By default output "pre" objects;
 
 sub pre_fetch {
     my $self = shift;
-    @{ $self->__pre_childs };
+    @{$self->__pre_childs};
 }
 
 =head2 post_fetch ($session)
@@ -174,7 +175,7 @@ Output data follow to fetch method. By default output "post" objects;
 
 sub post_fetch {
     my $self = shift;
-    @{ $self->__post_childs };
+    @{$self->__post_childs};
 }
 
 =head1 OTHER
@@ -189,46 +190,56 @@ sub _set_parent {
     }
 }
 
-sub __any_path {
-    my $self = shift;
-    my $sess = shift;
-    my ( $method ) = @_;
-    my ( $res, $path ) = $self->SUPER::__any_path( $sess, @_ );
-    return undef unless defined($res);
-    if ( ref($res) eq 'ARRAY' ) {
-
-        #make container
-        my $cont = $self->__engine->_create_( $method, __PACKAGE__ );
-        $cont->_set_childs_(@$res);
-        $res = [$cont];
-        unshift( @$path, $method );
-    }
-    return ( $res, $path );
-}
-
 #Return (  object witch handle req and result )
 sub _traverse_ {
     my $self = shift;
     my $sess = shift;
+
     #if empty path return $self
     unless ( scalar(@_) ) { return ( $self, $self ) }
 
+    #extra path need only then seek object
+    #no __extra_path if resolve
+
+=pod
+    #first check if exists
+    #strip extra path from urls
+    if ( my $epath = $self->__extra_path ) {
+
+        # not found if extra path greater when @path
+        #hint: /elem (extra_path 2010/123) and path only "2010"
+        return undef if ( scalar(@$epath) > scalar(@_) );
+
+        #check if extra eq
+        my $not_eq = 0;
+        my $pos    = 0;
+        for (@$epath) {
+            next if $_[ $pos++ ] eq $_;
+            $not_eq++;
+            last;
+        }
+        #check if ok with extra path
+        return undef if $not_eq;    # not found
+        #cut extra path from path
+        splice @_, 0, scalar(@$epath);
+        #if empty path return $self
+        unless ( scalar(@_) ) { return $self }
+    }
+=cut
+
     my ( $next_name, @path ) = @_;
-    
-    #$src - object wich handle answer, $res - answer
-    my ($src, $res ) = ($self, undef);
+
     #check if exist object with some name
     if ( my $obj = $self->_get_obj_by_name($next_name) ) {
 
         #if last in path return him
-        ($src, $res ) =  $obj->_traverse_( $sess, @path );
+        return $obj->_traverse_( $sess, @path );
 
     }
     else {    #try get other ways
 
         #try get objects by special methods
-        my $last_path;
-        ( $res, $last_path ) = $self->__any_path( $sess, $next_name, @path );
+        my ( $res, $last_path ) = $self->__any_path( $sess, $next_name, @path );
         return ( $self, undef ) unless defined $res;    #break search
         if ( UNIVERSAL::isa( $res, 'WebDAO::Response' ) ) {
             return ( $self, $res );
@@ -250,22 +261,19 @@ sub _traverse_ {
             }
         }
         if (@rest_path) {
-            ($src, $res ) =  $self->_traverse_( $sess, @rest_path );
+            return $self->_traverse_( $sess, @rest_path );
         }
+
+        #unknown $res
+        # <WebDAO::Element>, may be HASH ref, STRING
+        return ( $self, $res );
     }
-    #
-    if ($src == $res && ! UNIVERSAL::isa( $src, 'WebDAO::Modal' ) ) {
-        #force set root object for Modal
-        $src = $res = $self if UNIVERSAL::isa( $self, 'WebDAO::Modal' )
-    }
-    return ( $src, $res )
+
 }
 
 #deprecated
 sub _call_method {
     my $self = shift;
-        _deprecated $self "_traverse_";
-
     my ( $name, @path ) = @{ shift @_ };
     return $self->SUPER::_call_method( [ $name, @path ], @_ ) || do {
         if ( my $obj = $self->_get_obj_by_name($name) ) {
@@ -324,6 +332,132 @@ sub _destroy {
     $self->SUPER::_destroy;
 }
 
+=head2 _get_object_by_path <$path>, [$session]
+
+Return first Element object for path.
+Try to load objects for current object.
+
+=cut
+
+#deprecated
+sub _get_object_by_path {
+    my $self    = shift;
+    my $path    = shift;
+    my $session = shift;
+
+    #    _log1 $self Dumper {'$self'=>ref($self), path=>$path};
+    my @backup_path = @$path;
+    my $next_name   = $path->[0];
+
+    #first try get by name
+    if ( my $obj = $self->_get_obj_by_name($next_name) ) {
+        shift @$path;    #skip first name
+                         #ok got it
+                         #check if it container
+                         #skip extra path
+        if ( UNIVERSAL::can( $obj, '__extra_path' ) ) {
+            my $extra_path = $obj->__extra_path;
+
+            #if extra path defined and not ref convert to ref
+            if ( defined $extra_path ) {
+                $extra_path = [$extra_path] unless ref($extra_path);
+            }
+            if ( ref($extra_path) ) {
+                my @extra = @$extra_path;
+
+                #now skip extra
+                for (@extra) {
+                    if ( $path->[0] eq $_ ) {
+                        shift @$path;
+                    }
+                    else {
+                        _log2 $self "Break __extra_path "
+                          . $path->[0] . " <> "
+                          . $_
+                          . " for : $obj";
+                        last;
+                    }
+                }
+            }
+        }
+        if ( $obj->isa('WebDAO::Container') ) {
+            return $obj unless @$path;    # return object if end of path
+            return $obj->_get_object_by_path( $path, $session );
+        }
+        else {
+
+            #if element return point in any way
+            return $obj;
+        }
+    }
+    else {
+
+        #try get objects by special methods
+        my $dyn = $self->__get_objects_by_path( $path, $session )
+          || return;    #break search
+
+        #handle self controlled objects
+        if ( $dyn eq $self ) {
+            return $self;
+        }
+        $dyn = [$dyn] unless ref($dyn) eq 'ARRAY';
+
+        #now try find object in returned array
+        my $next;
+        foreach (@$dyn) {
+
+            #skip non objects
+            next unless $_->_obj_name eq $next_name;
+            $next = $_;
+            last;    #exit from loop loop
+        }
+        unless ($next) {
+            return    # return undef unless find objects
+        }
+        else {
+
+            # yes, from returned object present traverse continue
+            #if defined $session ( load scene)
+            if ($session) {
+                $self->_add_childs(@$dyn);
+                return $self->_get_object_by_path( $path, $session );
+            }
+            else {
+
+                #if query without session
+                #try to find  by name
+                #ok got it
+                #check if it container
+                if ( $next->isa('WebDAO::Container') ) {
+                    return $next->_get_object_by_path( $path, $session );
+                }
+                else {
+
+                    #return object referense in any way
+                    return $next;
+                }
+            }
+
+        }
+    }
+}
+
+=head2 __get_objects_by_path [path], $session
+
+Return next object for path 
+
+=cut
+
+#deprecateed
+sub __get_objects_by_path {
+    my $self = shift;
+    my ( $path, $session ) = @_;
+
+    # check if path point to method
+    return $self if $self->can( $path->[0] );
+    return;    # default return undef
+}
+
 1;
 __DATA__
 
